"""Module for defining our data naming conventions"""

from datetime import datetime
from enum import Enum


class RegexParts(str, Enum):
    """Regular expression components to be re-used elsewhere"""

    DATE = r"\d{4}-\d{2}-\d{2}"
    TIME = r"\d{2}-\d{2}-\d{2}"
    DATETIME = r"\d{4}-\d{2}-\d{2}_\d{2}-\d{2}-\d{2}"


class DataRegexLegacy(str, Enum):
    """Deprecated regular expression patterns from aind-data-schema-models v1 and earlier"""

    # Deprecated patterns, keeping for legacy support
    DATA = f"^(?P<label>.+?)_(?P<c_date>{RegexParts.DATE.value})_(?P<c_time>{RegexParts.TIME.value})$"
    RAW = (
        f"^(?P<platform_abbreviation>.+?)_(?P<subject_id>.+?)_(?P<c_date>{RegexParts.DATE.value})_(?P<c_time>"
        f"{RegexParts.TIME.value})$"
    )
    DERIVED = (
        f"^(?P<input>.+?_{RegexParts.DATE.value}_{RegexParts.TIME.value})_(?P<process_name>.+?)_(?P<c_date>"
        f"{RegexParts.DATE.value})_(?P<c_time>{RegexParts.TIME.value})"
    )
    ANALYZED = (
        f"^(?P<project_abbreviation>.+?)_(?P<analysis_name>.+?)_(?P<c_date>"
        f"{RegexParts.DATE.value})_(?P<c_time>{RegexParts.TIME.value})$"
    )


class DataRegex(str, Enum):
    """Regular expression patterns for different kinds of data and their properties"""

    DATA = f"^(?P<label>.+?)_(?P<c_datetime>{RegexParts.DATETIME.value})$"
    RAW = f"^(?P<subject_id>.+?)_(?P<c_datetime>{RegexParts.DATETIME.value})$"
    DERIVED = (
        f"^(?P<input>.+?_{RegexParts.DATETIME.value})_(?P<process_name>.+?)_(?P<c_datetime>"
        f"{RegexParts.DATETIME.value})"
    )
    ANALYZED = f"^(?P<project_abbreviation>.+?)_(?P<analysis_name>.+?)_(?P<c_datetime>" f"{RegexParts.DATETIME.value})$"
    NO_UNDERSCORES = r"^[^_]+$"
    NO_SPECIAL_CHARS = r'^[^<>:;"/|? \\_]+$'
    NO_SPECIAL_CHARS_EXCEPT_SPACE = r'^[^<>:;"/|?\\_]+$'


class DataLevel(str, Enum):
    """Data level name"""

    DERIVED = "derived"
    RAW = "raw"
    SIMULATED = "simulated"


class Group(str, Enum):
    """Data collection group name"""

    BEHAVIOR = "behavior"
    EPHYS = "ephys"
    MSMA = "MSMA"
    OPHYS = "ophys"
    NBA = "NBA"


def datetime_to_name_string(dt: datetime) -> str:
    """
    Take a datetime object, format it as a legacy AIND datetime string

    Parameters
    ----------
    dt : datetime
      For example, datetime(2020, 12, 29, 10, 04, 59)

    Returns
    -------
    str
      For example, '2020-12-29_10-04-59'

    """
    return dt.strftime("%Y-%m-%d_%H-%M-%S")


def datetime_from_name_string(dt: str) -> datetime:
    """
    DEPRECATED: Use datetime.fromisoformat() instead, except on Python 3.10

    Take datetime string, generate datetime object

    Returns
    -------
    datetime

    """
    return datetime.strptime(dt, "%Y-%m-%d_%H-%M-%S")


def build_data_name(label: str, creation_datetime: datetime) -> str:
    """
    Construct a data description name from a label and datetime object
    Parameters
    ----------
    label : str
      For example a subject_id, '123456'
    creation_datetime : datetime
      For example, datetime(2020, 12, 29, 10, 04, 59)

    Returns
    -------
    str
      For example, '123456_2020-12-29_10-04-59'

    """
    dt_str = datetime_to_name_string(creation_datetime)
    return f"{label}_{dt_str}"



"""Unit tests for the Gene model, focusing on GenBank accession ID retrieval."""

import unittest
from pathlib import Path
from unittest.mock import Mock, patch

from aind_data_schema_models.gene import Gene, NucleotideModel


class TestGene(unittest.TestCase):
    """Tests for Gene model"""

    def setUp(self):
        """Setup"""
        # Load the mock GenBank response from file
        resource_path = Path(__file__).parent / "resources" / "genbank_response.txt"
        with open(resource_path, "r") as f:
            self.mock_genbank_text = f.read()

    @patch("aind_data_schema_models.gene.requests.get")
    def test_from_genbank_accession_id(self, mock_get):
        """Test fetching nucleotide data from GenBank accession ID."""
        # Setup mock response
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.text = self.mock_genbank_text
        mock_get.return_value = mock_response

        accession_id = "LN515608"
        nucleotide = Gene.from_genbank_accession_id(accession_id)
        self.assertIsInstance(nucleotide, NucleotideModel)
        self.assertIn("Synthetic construct for Aequorea victoria partial gfp gene for GFP.", nucleotide.description)
        self.assertEqual(nucleotide.registry_identifier, "LN515608")
        self.assertEqual(nucleotide.registry.name, "GENBANK")
        self.assertEqual(nucleotide.name, "gfp")

    @patch("aind_data_schema_models.gene.requests.get")
    def test_from_genbank_accession_id_blank_response_raises(self, mock_get):
        """Test that a blank GenBank response raises a ValueError."""
        # Setup mock response with blank text
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.text = ""
        mock_get.return_value = mock_response

        accession_id = "LN000000"
        with self.assertRaises(ValueError):
            Gene.from_genbank_accession_id(accession_id)

    @patch("aind_data_schema_models.gene.requests.get")
    def test_from_genbank_accession_id_missing_gene_name_raises(self, mock_get):
        """Test that missing gene name in GenBank response raises a ValueError."""
        # Setup mock response with DEFINITION but no gene name
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.text = (
            "DEFINITION  Synthetic construct for Aequorea victoria partial gfp gene for GFP."
            "\nACCESSION   LN515608\n"  # No gene="..." in text
        )
        mock_get.return_value = mock_response

        accession_id = "LN515608"
        with self.assertRaises(ValueError):
            Gene.from_genbank_accession_id(accession_id)


if __name__ == "__main__":
    unittest.main()



"""Test reagent enums"""

import unittest

from aind_data_schema_models.reagent import FluorophoreType


class UnitsTests(unittest.TestCase):
    """Class for testing reagent enums"""

    def test_units(self):
        """Tests creation of a FluorophoreType object"""

        self.assertIsNotNone(FluorophoreType.ALEXA)


if __name__ == "__main__":
    unittest.main()



"""Tests classes in process_names module"""

import unittest

from aind_data_schema_models.process_names import ProcessName


class TestProcessName(unittest.TestCase):
    """Tests methods in ProcessName class"""

    def test_class_construction(self):
        """Tests enum can be instantiated via string"""

        self.assertEqual(ProcessName.COMPRESSION, ProcessName("Compression"))


if __name__ == "__main__":
    unittest.main()
